@@ -34,12 +34,21 @@ from .. import bootloader, utils
 from ..lvm import LVM
 from ..naming import Image
 from ..utils import mounted, ShellVarFile, RpmPackageDb, copy_files, Fstab,\
-    File, SystemRelease, Rsync, kernel_versions_in_path, IDMap, remove_file
+    File, SystemRelease, Rsync, kernel_versions_in_path, IDMap, remove_file, \
+    find_mount_target
 
 
 log = logging.getLogger(__package__)
 
 
+class SeparateVarPartition(Exception):
+    pass
+
+
+class BootPartitionRequires1G(Exception):
+    pass
+
+
 class ConfigMigrationError(Exception):
     pass
 
@@ -76,6 +85,12 @@ def on_new_layer(imgbase, previous_lv, new_lv):
         log.exception("Failed to migrate etc")
         raise ConfigMigrationError()
 
+    if not os.path.ismount("/var"):
+        raise SeparateVarPartition(
+            "\nIt's required /var as separate mountpoint!"
+            "\nPlease check documentation for more details!"
+        )
+
     try:
         adjust_mounts_and_boot(imgbase, new_lv, previous_layer_lv)
     except:
@@ -101,6 +116,33 @@ def on_post_init_layout(imgbase, existing_lv, new_base, new_layer):
     new_etc.mount()
 
 
+def boot_partition_validation():
+    """
+    Function to validate all requirements for /boot partition
+    """
+    boot_dir = None
+    bytes_in_1G = 1000**3
+
+    for target in find_mount_target():
+        if "boot" in target:
+            boot_dir = target
+            break
+
+    if boot_dir is None:
+        raise RuntimeError("findmnt: error, unable to find boot partition"
+                           " in target!")
+
+    st = os.statvfs(boot_dir)
+    bytes_in_boot_partition = st.f_blocks * st.f_frsize
+
+    if bytes_in_boot_partition < bytes_in_1G:
+        # 1G is 1073741824 bytes. However, if users use size=1000
+        # in anaconda kickstart won't work. Based on that, let's
+        # inform to users it's required 1.1G (size=1100).
+        log.error("New /boot must have at least 1.1G size")
+        raise BootPartitionRequires1G
+
+
 def migrate_etc(imgbase, new_lv, previous_lv):
     log.debug("Migrating etc (%s -> %s)" % (previous_lv, new_lv))
     with mounted(new_lv.path) as new_fs,\
@@ -114,8 +156,7 @@ def migrate_etc(imgbase, new_lv, previous_lv):
         log.info("Verifying stream compatability")
         log.debug("%s vs %s" % (old_rel, new_rel))
 
-        if new_rel.PRODUCT not in ["fedora", "centos",
-                                   "enterprise_linux"]:
+        if not new_rel.is_supported_product():
             log.error("Unsupported product: %s" % new_rel)
 
         is_same_product = old_rel.PRODUCT == new_rel.PRODUCT
@@ -162,6 +203,7 @@ def migrate_etc(imgbase, new_lv, previous_lv):
 
         log.info("Syncing systemd levels")
         fix_systemd_services(old_fs, new_fs)
+        relocate_var_lib_yum(new_fs)
 
         hack_rpm_permissions(new_fs)
 
@@ -193,7 +235,27 @@ def fix_systemd_services(old_fs, new_fs):
 
     for d in diffs:
         log.debug("Removing %s" % d)
-        remove_file(new_fs.path("/") + d)
+        try:
+            if os.path.exists(new_fs.path("/") + d):
+                if os.path.isdir(new_fs.path("/") + d):
+                    remove_file(new_fs.path("/") + d, dir=True)
+                elif os.path.isfile(new_fs.path("/") + d):
+                    remove_file(new_fs.path("/") + d)
+        except:
+            log.exception("Could not remove %s. Is it a read-only layer?")
+
+
+def relocate_var_lib_yum(new_fs):
+    path = "/var/lib/yum"
+    # Check whether /var is a symlink to /usr/share, and move it if it is not
+    # We could directly check this in new_fs, but this gets tricky with
+    # symlinks, and it will already be present on new builds
+    if not os.path.islink(path):
+        log.debug("/var/lib/yum is not a link -- moving it")
+        shutil.rmtree(path)
+        os.mkdir(path)
+        shutil.move(path, "/usr/share/yum")
+        os.symlink("/usr/share/yum", "/var/lib/yum")
 
 
 def hack_rpm_permissions(new_fs):
@@ -353,6 +415,18 @@ def adjust_mounts_and_boot(imgbase, new_lv, previous_lv):
         initrd = "/boot/initramfs-%s.img" % kver
         chroot("dracut", "-f", initrd, "--kver", kver)
 
+        # Copy the .hmac file for FIPS until rhbz#1415032 is resolved
+        # Since .hmac is a plain checksum pointing at a bare path in /boot,
+        # we need to copy everything
+        with utils.bindmounted("/boot", newroot + "/boot"):
+            log.debug("Copying FIPS files")
+            files = glob.glob("/boot/%s/*" % new_lv.lv_name) + \
+                    glob.glob("/boot/%s/.*" % new_lv.lv_name)
+            log.debug(files)
+            for f in files:
+                log.debug("Copying %s to /boot" % f)
+                shutil.copy2(f, "/boot")
+
     def __check_kernel_files(pkgfiles, newroot):
         kfiles = ["%s/%s" % (newroot, f)
                   for f in pkgfiles
@@ -442,6 +516,11 @@ def adjust_mounts_and_boot(imgbase, new_lv, previous_lv):
             copy_kernel(newroot.target)
             add_bootentry(newroot.target)
 
+            try:
+                boot_partition_validation()
+            except:
+                raise
+
     imgbase.hooks.emit("os-upgraded",
                        previous_lv.lv_name,
                        new_lvm_name)
